<?php
namespace Controller;

use \Service\Release as ReleaseService;
use \Service\Package as PackageService;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row as RequestRow;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;

class Release extends \Generated\Controller\Release {
    /**
     * @var ReleaseService
     */
    protected $releaseService;
    /**
     * @var PackageService
     */
    protected $packageService;
    /** @var Environment  */
    protected $environment;

    /**
     * @param Manager $dataSourceManager
     * @param Environment $environment
     * @param ReleaseService $releaseService
     * @param PackageService $packageService
     */
    public function __construct(Manager $dataSourceManager, Environment $environment, ReleaseService $releaseService, PackageService $packageService) {
        $this->environment = $environment;
        $this->releaseService = $releaseService;
        $this->packageService = $packageService;
        parent::__construct($dataSourceManager);
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::save($request, $response, $showMessage);
    }

    /**
     * @param DTO\Request $request
     * @param Response $response
     * @param bool $showMessage
     */
    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }



    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveRelease(RequestDataset $request, Response $response) {
        try {
            $releaseData   = $request->getDataSet()->getRows();
            $releaseData   = $releaseData[0];
            $id            = $releaseData['ID'];
            $name          = $releaseData['NAME'];
            $productId     = $releaseData['PRODUCT_ID'];
            $userId        = empty($releaseData['ZH_USER']) ? $releaseData['USER_ID'] : $releaseData['ZH_USER'];
            $productServer = $releaseData['PRODUCT_SERVER_ID'];
            $updateVersion = $releaseData['UPDATE_VERSION'];
            $fixRelease    = $releaseData['IS_FIX_RELEASE'];
            $status        = $releaseData['STATUS'];
            $version       = $releaseData['VERSION'];
            $revision      = $releaseData['REVISION'];
            $build         = $releaseData['BUILD'];
            $description   = $releaseData['DESCRIPTION'];
            $isActive      = $releaseData['IS_ACTIVE'];

            $idSaved = $this->releaseService->saveRelease($id, $name, $productId, $userId, $productServer, $updateVersion,
                                               $fixRelease, $status, $version, $revision, $build, $description, $isActive);
            $response->addDataSet(new DataSet('release', array($idSaved)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function removePackage(RequestDataset $request, Response $response) {
        try {
            $releaseData = $request->getDataSet()->getRows();
            $releaseData = $releaseData[0];
            $packageId   = $releaseData['ID'];
            $this->releaseService->removePackage($packageId);

            if(!empty($releaseData['PRODUCT_ID']) && !empty($releaseData['UPDATE_RELEASES'])) {
                $this->packageService->updateReleasePackagesByProduct($releaseData['PRODUCT_ID']);
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
      public function inactiveReleasesFix(RequestFilter $request, Response $response){
       
        $result     = $this->releaseService->inactiveReleasesFix();
        $response->addDataSet(new DataSet('inactiveReleasesFIx', [$result]));
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function removePathRelease(RequestFilter $request, Response $response) {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $subReleaseId = Utilities::getValueInFilterDataSourceByName($filters,'RELEASE_ID');
            $this->releaseService->removePathReleaseForController($subReleaseId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findPackagesByRelease(RequestFilter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $releaseId  = empty($conditions[0]) ? 0 : $conditions[0]['value'];
            $packages   = $this->releaseService->findPackagesByRelease($releaseId);
            $response->addDataSet(new DataSet('packages', $packages));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }


    public function checkPermission(RequestFilter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $releaseId  = Utilities::getValueInFilterDataSourceByName($conditions, 'RELEASE_ID');
            $result     = $this->releaseService->checkPermission($releaseId);
            $response->addDataSet(new DataSet('checkPermission', [$result]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function homologateRelease(RequestDataset $request, Response $response)
    {
        try {
            $releaseData = $request->getDataSet()->getRows();
            $params = $releaseData[0];
            $releaseId = $params['ID'];
            $version = $params['VERSION'];
            $revision = $params['REVISION'];
            $build = $params['BUILD'];
            $releaseStatus = $params['RELEASE_STATUS'];
            $subReleaseStatus = $params['SUBRELEASE_STATUS'];
            $isProjectAssociation = $params['IS_PROJECT_ASSOCIATION'];
            $packages = $params['PACKAGES'];
            $modulesConfiguration = $params['MODULES_CONFIGURATION'];

            $this->releaseService->homologateRelease($releaseId, $version, $revision, $build, $releaseStatus,
                $subReleaseStatus, $isProjectAssociation, $packages, $modulesConfiguration);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findPackageServersByProduct(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $productId  = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        $filter = Utilities::getFilterTableColumn($conditions, 'SERVER_NAME');
        try {
            $registers = $this->releaseService->findPackageServersByProduct($productId, $filter);
            $response->addDataSet(new DataSet('serversByProductAndType', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function validateReleaseServerMnemonic(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $releaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'RELEASE');
            $typeEvent = Utilities::getValueInFilterDataSourceByName($conditions, 'TYPE_EVENT');
            $productFiles = $this->releaseService->validateReleaseServerMnemonic($releaseId, $typeEvent);
            $response->addDataSet(new DataSet('validateReleaseServerMnemonic', $productFiles));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeRelease(RequestRow $request, Response $response) {
        try {
            $release = $request->getRow();
            $releaseId = $release['ID'];

            $this->releaseService->removeRelease($releaseId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findRelease(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        try {
            $registers = $this->releaseService->findRelease($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('release', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function getTaskInfoForGenerate(RequestFilter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $resourceId = Utilities::getValueInFilterDataSourceByName($conditions, 'RESOURCE_ID');
            $releaseId  = Utilities::getValueInFilterDataSourceByName($conditions, 'RELEASE_ID');
            $registers  = $this->releaseService->getTaskInfoForGenerate($resourceId, $releaseId);
            $response->addDataSet(new DataSet('taskInfo', $registers));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findResponsesOfIssuesAndTasksByRelease(RequestFilter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $releaseId  = Utilities::getValueInFilterDataSourceByName($conditions, 'RELEASE_ID', true);
            $issuesResponse = $this->releaseService->findResponsesOfIssuesAndTasksByRelease($releaseId, $conditions);
            $response->addDataSet(new DataSet('findResponsesOfIssuesAndTasksByRelease', $issuesResponse));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setPackageCreatedStatusInRelease(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $releaseId = $row['RELEASE_ID'];
            $this->releaseService->setPackageCreatedStatusInSubRelease($releaseId);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setStatusInRelease(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $releaseId = $row['RELEASE_ID'];
            $status = $row['STATUS'];
            $this->releaseService->setStatusInRelease($releaseId, $status);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}